# J Norman's model for inversion of LAI
# coded by Hideki Kobayashi 
# Feb 8 2011
# modified Feb 18, 2012

LAI2200C_model_inversion <- function()
{
	
rad = pi / 180.0

# make output array 
output = diag(0, nrow = 23, ncol = 1)
# there are two trials.
# in the first trial, scattering effect will not be considered
# in the second trial, scattering effect will be considered to estimate LAI
for(n in (1:2)){

# file code
file1 = paste("Input")
file_gap = paste(file1,".gap",sep="")
file_hdr = paste(file1,".hdr",sep="")
file_sky = paste(file1,".sky",sep="")
header <- read.table(file_hdr)
header =t(header)
sky <- read.table(file_sky)
gapdata <- read.table(file_gap)

# LAI-2200 readings
# for FOV45
LI22FOV45 = diag(0,nrow=6,ncol=1)
LI22FOV45[1] = sky[[1,1]]
LI22FOV45[2] = sky[[1,2]]
LI22FOV45[3] = sky[[1,3]]
LI22FOV45[4] = sky[[1,4]]
LI22FOV45[5] = sky[[1,5]]
LI22FOV45[6] = sky[[1,5]]

# for FOV270
LI22FOV270=diag(0,nrow=6,ncol=1)
LI22FOV270[1] = sky[[2,1]]
LI22FOV270[2] = sky[[2,2]]
LI22FOV270[3] = sky[[2,3]]
LI22FOV270[4] = sky[[2,4]]
LI22FOV270[5] = sky[[2,5]]
LI22FOV270[6] = sky[[2,5]]

# get haeder info
gdata = gapread(gapdata) # read gap data and first estimate of omega and L
	
# L = 1.0	
if(n == 1){	        # total leaf area index
L = unlist(gdata[1])
}
omega = unlist(gdata[2])    # mean clumping
omega_th = unlist(gdata[3])
mean_gaps = unlist(gdata[4])
gaps = as.matrix(data.frame(gdata[5]))

if(n == 1){	
cat("LAI-2000 algorithm\n")
cat("Orig L=",L)
cat("  ")
cat("m_omega=",omega,"\n")
cat("omega=",omega_th,"\n")

# transfer to output array
output[1] = L
output[2] = omega
output[3:7] = omega_th
ncnt = 7
}

# Clumping index omega for 5 rings
# omega_th=diag(0,nrow=5,ncol=1)
# omega_th[1] = 0.5
# omega_th[2] = 0.5
# omega_th[3] = 0.5
# omega_th[4] = 0.5
# omega_th[5] = 0.5

# average clumping index omega
# omega = mean(omega_th)

## average clumping index omega
# omega = mean(omega_th)
#rou = 0.0535          # single leaf reflectance (at Tonzi)
#tau = 0.0128          # single leaf transmittance (at Tonzi)

#rou = 0.09315          # single leaf reflectance with 50 % mixture of stem (at Tonzi)
#tau = 0.0064          # single leaf transmittance with 50 % mixture of stem (at Tonzi)

#rou = 0.0723          # single leaf reflectance with 33 % mixture of stem (at Tonzi)
#tau = 0.0083          # single leaf transmittance with 33 % mixture of stem (at Tonzi)

#rou = 0.00          # Black leaves
#tau = 0.00          # Black leaves

rou = as.numeric(header[1])
tau = as.numeric(header[2])

rous = 0.1

# leaf refractive index
refln = 1.3
rouSP = ((1 - refln)/(1 + refln)) ^2

absrp = 1.0 - rou - rouSP - tau # leaf absorption

Fbm = as.numeric(header[6])          # beam fraction
Fdif = 1.0 - Fbm    # diffuse fraction

theS = as.numeric(header[3])         # solar zenith angle
theV = c(7, 22, 37, 53, 67)          # View zenith angle Here we fixed it for LAI2000 view (centre)
theV = 180.0 - theV          # should be oppisite to express the view is oppsite direction to sun

phiS = 0.0          # solar azimuth angle
phiS = 180.0 - phiS   # since theV vector I sued is oppisite direction (180-theV), so phiS should also be an oppitite
phiV = abs(as.numeric(header[4])-as.numeric(header[5]))        # View azimuth angleｓ

# change these angles to radian
theS = pi * theS / 180.0         # solar zenith angle
theV = pi * theV / 180.0         # View zenith angle
phiS = pi * phiS / 180.0         # solar azimuth angle
phiV = pi * phiV / 180.0         # View azimuth angle


#LI22FOV270 should be devided by 6 to adjust the FOV45 area 270/45 = 6
# Here I added an 6th virtual view angle to cover the whole sphere 
theV=c(theV,83.0*rad)
denom=(LI22FOV270/6) * sin(theV) * abs(cos(theV)) * 15 * rad
# remove 6th ring to avoid confusion
theV = theV[1:5]

# computing the Isky
# this was wrong equation we found. we don't need pi
#Isky = Fdif * LI22FOV45 / (2*pi*sum(denom))
Isky = Fdif * LI22FOV45 / (2*sum(denom))
#cat("Isky=",Isky,"\n")	

# integration of Ileaf from LAI 0 to L	
dL = 0.1           # the layer thickness
nlay = floor(L / dL)   # number of layer
Ileaf = 0.0
Ileaf_sun = 0.0
Ileaf_sh = 0.0

# make LAI array
LAIarrm = c(0:(nlay-1)) * dL
LAIarr = c(1:nlay) * dL


# parameter declaration

mean_ln_gaps = diag(0, nrow = 5, ncol = 1)
mean_gaps = diag(0, nrow = 5, ncol = 1)
sbm=diag(0, nrow=5, ncol=1)
sdif=diag(0, nrow=5, ncol=1)
Ileaf_sun = diag(0, nrow=5, ncol=nlay)
Ileaf_sh = diag(0, nrow=5, ncol=nlay)
Ileaf_refl = diag(0, nrow=5, ncol=nlay)
Ileaf = diag(0, nrow=5, ncol=1)
gaps_corr=diag(0, nrow=nrow(gaps), ncol=5)
Fview = diag(0, nrow = 5, ncol = nlay)


##########################
# start iteration
icount = 1
# initial evaluation function F
# should be minus for the increment of kai
F = -0.1
# initial guess of kai
kai = 1.0          # leaf angle distribution controll parameter

idx = 0.2
imax = 300
while (icount < imax){

dx = -((idx - 0.001) / (imax-5)) * icount + idx          # increment for kai
dx = max(dx, 0.001)
# should not use loop to avoid slow execution of R

# Kbe extinction coefficient for elliptical leaf angle distribution
KbeS = sqrt(kai * kai + tan(theS) * tan(theS))
KbeS = KbeS / (kai + 1.774 * (kai + 1.182)^-0.733)

KbeV = sqrt(kai * kai + tan(theV) * tan(theV))
KbeV = KbeV / (kai + 1.774 * (kai + 1.182)^-0.733)

# Kde: extinction for diffuse radiation
# set theta angle
the = c(0:90) * rad
Kbearr = sqrt(kai * kai + tan(the) * tan(the))
Kbearr = Kbearr / (kai + 1.774 * (kai + 1.182)^-0.733)
sinth = sin(the)
costh = cos(the)

# Transmission function for diffuse sky radiation
# Td = (1/2pi)Int(int(exp(- sqrt(myu) x Kbe x L)cos(the)sin(the)))dth dphi
Td = diag(0, nrow = nlay, ncol = 1)
Intfact = diag(0, nrow = nlay, ncol = 91)

for(i in 1:nlay){
Intfact[i,] = exp(- sqrt(absrp) * Kbearr * LAIarr[i] * omega) * sinth * costh
Td[i] = 2*rad * (0.5 * (Intfact[i,1] + Intfact[i,91]) + sum(Intfact[i,2:90]))
}


# Ishade
Ish = Fdif * Td + Fbm * (exp(-sqrt(absrp) * KbeS * omega * LAIarr) - exp(-KbeS * omega * LAIarr))

# Isun
# Fbm is a beam radiation relative to the leaf 
# This is still questionable for me Feb 4 2011, 
# but I got the same resutls as John's excel sheets
# when I used this.
#Fbmp = Fbm / cos(theS)

#Idir = Fbm / cos(theS)

# KbeS * cos(theS) is G-function
#Idir = Fbm * KbeS * cos(theS)
Idir = Fbm * KbeS
#Idir = Fbm / cos(theS)

#cat("KbeS=",KbeS,"\n")

#Idir = Fbm
#Fbmp = Fbm

# Isun is updated  June 27, 2010
Isun = Idir + Ish

# Fsun
Fsun = exp(-KbeS * LAIarrm * omega) - exp(-KbeS * LAIarr * omega)
Fsun = Fsun / (KbeS * dL)

# scattering coefficient for direct beam (sbm) and for diffuse (sdif)
# and Ileaf computation
#sdif = calsdif(kai, rou, tau)

# scattering coefficient for direct beam (sbm) and for diffuse (sdif)
# and Ileaf computation
for(i in (1:5)){

# calsdif4 is a exact form of sdif from integration of sbm
#sdif[i] = calsdif4(kai, rou, tau, theV[i]) *2/pi

# This is John's simple formula
sdif[i] = calsdif(kai, (rou+rouSP), tau)
# cat("sdif",sdif[i],"\n")
}


# reflected intensity from the soil/snow surface
Iflr = Fbm * exp (- sqrt(absrp) * KbeS * omega * L)
Iflr = Iflr + Fdif * Td[nlay]
Irefl = rous * Iflr * sort(Td)

for(i in 1:5){
# Fview for five rings	
Fview[i,] = exp(-KbeV[i] * omega_th[i] * (L - LAIarr)) - exp(-KbeV[i] * omega_th[i] * (L - LAIarrm))
# scattering coefficient	
sbm[i] = calsbm(theS, theV[i], phiS, phiV, kai, rou, tau, rouSP, refln, KbeS)
Ileaf_sun[i,] = Fsun * Idir * Fview[i,] * sbm[i] + Fsun * t(Ish) * Fview[i,] * sdif[i]
Ileaf_sh[i,] = (1 - Fsun) * t(Ish) * Fview[i,] * sdif[i] 
Ileaf_refl[i,] = Irefl * Fview[i,] * sdif[i]

# integration over LAI from 0 to L
Ileaf[i,] = sum(Ileaf_sun[i,]) + sum(Ileaf_sh[i,]) + sum(Ileaf_refl[i,])

}

# browser()

if(n == 1){
	Scat = 0.0 / Isky[1:5]
	}
else{
Scat = Ileaf / Isky[1:5]
}

# corrected P
 mean_gaps_corr = ((mean_gaps - Scat) > 0.001) * (mean_gaps - Scat) + ((mean_gaps - Scat) <= 0.001) * 0.001 
# This is a fualt equation. I put here to compare the correct one with this wrong one
# mean_gaps_corr = (((mean_gaps - Scat) / (1 - Scat)) > 0.001) * ((mean_gaps - Scat) / (1 - Scat)) + (((mean_gaps - Scat) / (1 - Scat)) <= 0.001) * 0.001 

# based onthe corrected gaps, new omega is derived.

for(i in 1:5){
gaps_corr[,i] = ((gaps[,i] - Scat[i]) > 0.001) * (gaps[,i] - Scat[i]) + ((gaps[,i] - Scat[i]) <= 0.001) * 0.001
# This is a fualt equation. I put here to compare the correct one with this wrong one
#gaps_corr[,i] = (((gaps[,i] - Scat[i])/(1 - Scat[i])) > 0.001) * ((gaps[,i] - Scat[i])/(1 - Scat[i])) + (((gaps[,i] - Scat[i])/(1 - Scat[i])) <= 0.001) * 0.001
 
}

## renew clumping index ####
# logarithm of each gap fraction	
ln_gaps = -log(gaps_corr)	

# mean of log gap fractions & mean gaps over the one set of measurements

for(i in 1:5){
mean_ln_gaps[i] = mean(ln_gaps[,i])
mean_gaps[i] = mean(gaps[,i])
}

# logarithm of mean gaps (mgaps)
ln_mean_gaps = -log(mean_gaps)

# apparent clumping index omega_th	
omega_th = 	ln_mean_gaps / mean_ln_gaps

# mean omega
omega = mean(omega_th)	

###end of renew of omega ###

# find a solution
ln_mean_gaps_corr = log(mean_gaps_corr)
ln_omega = ln_mean_gaps_corr / omega_th

kai2 = kai + dx
KbeVV = sqrt(kai2 * kai2 + tan(theV) * tan(theV))
KbeVV = KbeVV / (kai2 + 1.774 * (kai2 + 1.182)^-0.733)
#KbeVV = KbeVV / (1.47 + 0.450 * kai2 + 0.1223 * kai2^2 - 0.0130 * kai2^3 + 0.000509 * kai2^4)

dKbeV = KbeVV - KbeV 
 
# evaluation function 
S1 = ln_omega * KbeV
S2 = KbeV * KbeV
S3 = KbeV * dKbeV
S4 = ln_omega * dKbeV

F = sum(S2) * sum(S4) - sum(S1) * sum(S3)
L = -sum(S1) / sum(S2)

# decide whichever we should increase dx or decrease
flag = (F < 0) * 1 + (F >= 0) * (-1)
kai = kai + flag * dx
#cat("flag&kai=",flag,kai, "\n")
kai = min(kai, 10)
kai = max(kai, 0.1)

#cat(icount)
#cat("    ")
#cat("kai=",kai)
#cat("    ")
#cat("LAI=",L)
#cat("    ")
#cat("F=",F,"\n")


icount = icount + 1

} # end while	

if(n==1){
cat(" \n")	
cat("Uncorrected estimations \n")	
}
else{
cat(" \n")	
cat("Scattering correction \n")	
}

#cat("scat=",Scat,"\n")

cat("LAI=",L)
cat("    ")	
cat("m_omega=",omega)	
cat("    ")	
cat("kai=",kai)
cat("    ")
cat("F=",F,"\n")	

cat("omega=",omega_th,"\n")	

# transfer to output array
output[ncnt+1] = L
output[ncnt+2] = omega
output[(ncnt+3):(ncnt+7)] = omega_th
output[ncnt+8] = kai

ncnt = ncnt + 8
	
}	
	
# output 
file_out = paste(file1,".out",sep="")
write.table(t(output),file_out,sep="  ",col.names=FALSE,row.names=FALSE)

} # end of main




