# this is effective scattering coefficient for beam radiation
# it can be derived from single leaf reflectance and transmittance in which
# the transmittance or reflectted lights are weighted by the fraction of view observed from the view direction.
# Hideki Kobayashi Feb 9, 2011

# the equation is 
# sbm = (tau * int(Fs * Fv * gL) + rou * int(Fs * Fv * gL))/(int(Fs * Fs * gL))

# matrix version for R

calsbm <- function(theS, theV, phiS, phiV, kai, rou, tau, rouSP,refln,KbeS)
#calsbm <- function()
{
	
rad = pi / 180.0

# These are the test data sets to check this code
#theS = 30 * rad 
#theV = 140 * rad
#phiS = 0 * rad 
#phiV = 0 * rad 
#kai = 1.5
#rou = 0.05
#tau = 0.01

# integration step
# step should be integer!!
# now this is fixed value
step = 10
dx = pi *  step / 180.0

beta = c(0: (360/step))	* rad * step
alpha = c(0: (90/step)) * rad * step

# get leaf angle distribution
g = gL(alpha, kai)

# get fraction of leaf area
H = diag(0, nrow =(90/step + 1), ncol = 1)
Ht_th = diag(0, nrow = ((90/step) + 1), ncol = 1)
Hp_th = diag(0, nrow = ((90/step) + 1), ncol = 1)
Ht_ph = diag(0, nrow = ((360/step) + 1), ncol = 1)
Hp_ph = diag(0, nrow = ((360/step) + 1), ncol = 1)

Ht = 0.0
Hp = 0.0

for(i in 1:((360/step) + 1)){
#for(i in 1:1){

fleaf_sun = fleaf(theS, alpha, phiS, beta[i])
fleaf_view = fleaf(theV, alpha, phiV, beta[i]) 
H = fleaf_sun * fleaf_view * g
# integral over theta direction
Ht_th = (H <= 0) * abs(H)
Hp_th = (H > 0) * H
Ht_ph[i] = dx * (0.5 * (Ht_th[1]+Ht_th[(90/step) + 1])+sum(Ht_th[2:(90/step)]))
Hp_ph[i] = dx * (0.5 * (Hp_th[1]+Hp_th[(90/step) + 1])+sum(Hp_th[2:(90/step)]))

#browser()

}

Ht = dx * (0.5 * (Ht_ph[1] + Ht_ph[((360/step) + 1)]) + sum(Ht_ph[2:(360/step)]))
Hp = dx * (0.5 * (Hp_ph[1] + Hp_ph[((360/step) + 1)]) + sum(Hp_ph[2:(360/step)]))

# separate transmittance and reflectance terms

#     specular component
#     gmsp computation

#the_sp = pi - theS
#if(abs(phiS-phiV) < pi){
#phr_sp = abs(phiS-phiV) + pi
#}
#else{
#  phr_sp = abs(phiS-phiV) - pi
#}

the_sp = theS
phr_sp = abs(phiV - phiS)

alpha = cos(the_sp) * cos(theV)+ sin(the_sp) * sin(theV) * cos(phr_sp)
alpha = acos(alpha)
alpha = alpha
a0 = alpha / 2.0

#Nilson's notation

thetai = asin(sin(a0)/refln)

#     cat(alpha,a0,thetai,"\n")

ang1 = a0 - thetai
ang2 = a0 + thetai

ang1 = max(0.0001,abs(ang1))    
ang2 = max(0.0001,abs(ang2))

ang1 = sign(ang1) * min(1.57,abs(ang1))    
ang2 = sign(ang2) * min(1.57,abs(ang2))

Rper = sin(ang1)**2 / sin(ang2)^2
Rpal = tan(ang1)**2 / tan(ang2)^2

Ref = 0.5 * (Rper + Rpal)

#     reduction factor
a0 = max(0.01,abs(a0))    
a0 = sign(a0) * min(1.57,abs(a0))   

thetaQ = acos((cos(the_sp) + cos(theV))/(2. * cos(a0)))


# leaf hair index
k_hair_ada = 0.15
k_hair_aba = 0.15

if(thetaQ > (pi / 2.)) {
  KP = 0.5*exp(-(2. * k_hair_aba* tan(a0))/pi) * (theV - 0.5 *pi)/pi

}
else{ 
  
#  KP = 0.0
#  if((theV*180/pi)> 150){
  KP = exp(-(2. * k_hair_ada* tan(a0))/pi) * (theV - 0.5 *pi)/pi
#  KP = 0.0
#  }
  
}

gmsp = (1./8.) * Ref * KP * gL(thetaQ, kai)/sin(thetaQ)
#cat("Ref=",theV*180/pi,Ref,gmsp,"\n")

# For John's formula, pi should be multiplyed
sbmsp = pi * gmsp / (pi * (KbeS * cos(theS))^2)
#sbmsp = gmsp / (pi * (KbeS * cos(theS)))
# Specular reflectance effect

#print(Ht)
#print(Hp)

sbm = (Ht * tau + Hp * rou) / (Ht + Hp) + sbmsp
#sbm = (Ht * tau + Hp * rou) / (2.0 * pi)

#cat("sbm=",sbm,sbmsp,gmsp,"\n")

# debug command	
#browser()	
return(sbm)	
	
}