# This is data reading module for LAI-2000 gap fraction
# Hideki Kobayashi
# Since Feb 16, 2011
# Hideki Kobayashi

# this function read gap fraction data setes for 5 rings 
# and make an average of gap for each ring
# and compute apparent gap fraction
# also compute initial guess of LAIs by Welles and Norman 1991 

gapread <- function(gaps){
	
#gaps <- read.table("gapfraction.txt")

# logarithm of each gap fraction	
ln_gaps = -log(gaps)	
# mean of log gap fractions
mean_ln_gaps = diag(0, nrow = 5, ncol = 1)
for(i in 1:5){
mean_ln_gaps[i] = mean(ln_gaps[,i])
}

# mean gaps over the one set of measurements
mean_gaps = diag(0, nrow = 5, ncol = 1)
for(i in 1:5){
mean_gaps[i] = mean(gaps[,i])
}	

# logarithm of mean gaps (mgaps)
ln_mean_gaps = -log(mean_gaps)

# apparent clumping index omega_th	
omega_th = 	ln_mean_gaps / mean_ln_gaps
# mean omega
omega = mean(omega_th)	
	
# effective LAI from Welles and Norman method	
rad = pi / 180.0
W = c(0.034, 0.104, 0.160, 0.218, 0.484)	
S = c(cos(7.0*rad), cos(22.0*rad), cos(37.0*rad), cos(53.0*rad), cos(67.0*rad))

fact = -log(mean_gaps) * W * S
Leff = 2.0 * sum(fact)

# true LAI as a initial guess
L = Leff / mean(omega)

#cat("omega=",omega_th)	
#browser()	
	
return(list(L, omega, omega_th, mean_gaps, gaps))		
}



